-- Author      : Kurapica
-- Create Date : 2013/07/22
-- Change Log  :

-- Check Version
local version = 2
if not IGAS:NewAddon("IGAS.Widget.IFSecurePanel", version) then
	return
end

-------------------------------------
-- Secure Manager
-------------------------------------
do
	-- Manager Frame
	_IFSecurePanel_ManagerFrame = SecureFrame("IGAS_IFSecurePanel_Manager", IGAS.UIParent)
	_IFSecurePanel_ManagerFrame.Visible = false

	_IFSecurePanel_ManagerFrame:Execute[[
		Manager = self

		IFSecurePanel_Panels = newtable()
		IFSecurePanel_Cache = newtable()
		IFSecurePanel_Map = newtable()

		UpdatePanelSize = [=[
			local noForce = ...
			local panel = IFSecurePanel_Map[self] or self
			local elements = IFSecurePanel_Panels[panel]
			local count = 0

			if elements and not panel:GetAttribute("IFSecurePanel_KeepMaxSize") and ( not noForce or panel:GetAttribute("IFSecurePanel_AutoSize") ) then
				for i = #elements, 1, -1 do
					if elements[i]:IsShown() then
						count = i
						break
					end
				end

				if count ~= IFSecurePanel_Cache[panel] then
					IFSecurePanel_Cache[panel] = count

					local row
					local column
					local columnCount = panel:GetAttribute("IFSecurePanel_ColumnCount") or 99
					local rowCount = panel:GetAttribute("IFSecurePanel_RowCount") or 99
					local elementWidth = panel:GetAttribute("IFSecurePanel_ElementWidth") or 16
					local elementHeight = panel:GetAttribute("IFSecurePanel_ElementHeight") or 16
					local hSpacing = panel:GetAttribute("IFSecurePanel_HSpacing") or 0
					local vSpacing = panel:GetAttribute("IFSecurePanel_VSpacing") or 0
					local marginTop = panel:GetAttribute("IFSecurePanel_MarginTop") or 0
					local marginBottom = panel:GetAttribute("IFSecurePanel_MarginBottom") or 0
					local marginLeft = panel:GetAttribute("IFSecurePanel_MarginLeft") or 0
					local marginRight = panel:GetAttribute("IFSecurePanel_MarginRight") or 0

					if panel:GetAttribute("IFSecurePanel_Orientation") == "HORIZONTAL" then
						row = ceil(count / columnCount)
						column = row == 1 and count or columnCount
					else
						column = ceil(count / rowCount)
						row = column == 1 and count or rowCount
					end

					panel:SetWidth(column * elementWidth + (column - 1) * hSpacing + marginLeft + marginRight)
					panel:SetHeight(row * elementHeight + (row - 1) * vSpacing + marginTop + marginBottom)
				end
			end
		]=]
	]]

	_IFSecurePanel_RegisterPanel = [=[
		local panel = Manager:GetFrameRef("SecurePanel")

		if panel and not IFSecurePanel_Panels[panel] then
			IFSecurePanel_Panels[panel] = newtable()
		end
	]=]

	_IFSecurePanel_UnregisterPanel = [=[
		local panel = Manager:GetFrameRef("SecurePanel")

		if panel then
			IFSecurePanel_Panels[panel] = nil
			IFSecurePanel_Cache[panel] = nil
		end
	]=]

	_IFSecurePanel_RegisterFrame = [=[
		local panel = Manager:GetFrameRef("SecurePanel")
		local frame = Manager:GetFrameRef("SecureElement")

		if panel and frame then
			IFSecurePanel_Panels[panel] = IFSecurePanel_Panels[panel] or newtable()
			tinsert(IFSecurePanel_Panels[panel], frame)

			IFSecurePanel_Map[frame] = panel
		end
	]=]

	_IFSecurePanel_UnregisterFrame = [=[
		local panel = Manager:GetFrameRef("SecurePanel")
		local frame = Manager:GetFrameRef("SecureElement")

		IFSecurePanel_Map[frame] = nil

		if panel and frame and IFSecurePanel_Panels[panel] then
			for k, v in ipairs(IFSecurePanel_Panels[panel]) do
				if v == frame then
					return tremove(IFSecurePanel_Panels[panel], k)
				end
			end
		end
	]=]

	_IFSecurePanel_WrapShow = [[
		Manager:RunFor(self, UpdatePanelSize, true)
	]]

	_IFSecurePanel_WrapHide = [[
		Manager:RunFor(self, UpdatePanelSize, true)
	]]

	_IFSecurePanel_UpdatePanelSize = [=[
		local panel = Manager:GetFrameRef("SecurePanel")

		IFSecurePanel_Cache[panel] = nil

		Manager:RunFor(panel, UpdatePanelSize)
	]=]

	_IFSecurePanel_ClearCache = [=[
		local panel = Manager:GetFrameRef("SecurePanel")

		IFSecurePanel_Cache[panel] = nil
	]=]

	function RegisterPanel(self)
		_IFSecurePanel_ManagerFrame:SetFrameRef("SecurePanel", self)
		_IFSecurePanel_ManagerFrame:Execute(_IFSecurePanel_RegisterPanel)
	end

	function UnregisterPanel(self)
		_IFSecurePanel_ManagerFrame:SetFrameRef("SecurePanel", self)
		_IFSecurePanel_ManagerFrame:Execute(_IFSecurePanel_UnregisterPanel)
	end

	function RegisterFrame(self, frame)
		_IFSecurePanel_ManagerFrame:SetFrameRef("SecurePanel", self)
		_IFSecurePanel_ManagerFrame:SetFrameRef("SecureElement", frame)
		_IFSecurePanel_ManagerFrame:Execute(_IFSecurePanel_RegisterFrame)

		_IFSecurePanel_ManagerFrame:WrapScript(frame, "OnShow", _IFSecurePanel_WrapShow)
		_IFSecurePanel_ManagerFrame:WrapScript(frame, "OnHide", _IFSecurePanel_WrapHide)
	end

	function UnregisterFrame(self, frame)
		_IFSecurePanel_ManagerFrame:UnwrapScript(frame, "OnShow")
    	_IFSecurePanel_ManagerFrame:UnwrapScript(frame, "OnHide")

		_IFSecurePanel_ManagerFrame:SetFrameRef("SecurePanel", self)
		_IFSecurePanel_ManagerFrame:SetFrameRef("SecureElement", frame)
		_IFSecurePanel_ManagerFrame:Execute(_IFSecurePanel_UnregisterFrame)
	end

	function SecureUpdatePanelSize(self)
		_IFSecurePanel_ManagerFrame:SetFrameRef("SecurePanel", self)
		_IFSecurePanel_ManagerFrame:Execute(_IFSecurePanel_UpdatePanelSize)
	end

	function SecureClearPanelCache(self)
		_IFSecurePanel_ManagerFrame:SetFrameRef("SecurePanel", self)
		_IFSecurePanel_ManagerFrame:Execute(_IFSecurePanel_ClearCache)
	end
end

__Doc__[[IFSecurePanel provides features to build a secure panel to contain elements of same secure class in a grid, the elements are generated by the IFSecurePanel]]
interface "IFSecurePanel"
	extend "IFIterator"

	local function AdjustElement(element, self)
		if not element.ID then return end

		element.Width = self.ElementWidth
		element.Height = self.ElementHeight

		if self.Orientation == Orientation.HORIZONTAL then
			-- Row first
			element:SetPoint("CENTER", self, "TOPLEFT", (element.ID - 1) % self.ColumnCount * (self.ElementWidth + self.HSpacing) + self.MarginLeft + (self.ElementWidth/2), -floor((element.ID - 1) / self.ColumnCount) * (self.ElementHeight + self.VSpacing) - self.MarginTop - (self.ElementHeight/2))
		else
			-- Column first
			element:SetPoint("CENTER", self, "TOPLEFT", floor((element.ID - 1) / self.RowCount) * (self.ElementWidth + self.HSpacing) + self.MarginLeft + (self.ElementWidth/2), -((element.ID - 1) % self.RowCount) * (self.ElementHeight + self.VSpacing) - self.MarginTop - (self.ElementHeight/2))
		end
	end

	local function AdjustPanel(self)
		if self.KeepMaxSize then
			self.Width = self.ColumnCount * self.ElementWidth + (self.ColumnCount - 1) * self.HSpacing + self.MarginLeft + self.MarginRight
			self.Height = self.RowCount * self.ElementHeight + (self.RowCount - 1) * self.VSpacing + self.MarginTop + self.MarginBottom
		else
			SecureUpdatePanelSize(self)
		end
	end

	local function Reduce(self, index)
		index = index or self.RowCount * self.ColumnCount

		if index < self.Count then
			local ele

			for i = self.Count, index + 1, -1 do
				ele = self:GetChild(self.ElementPrefix .. i)
				self:Fire("OnElementRemove", ele)
				ele:Dispose()

				self:SetAttribute("IFSecurePanel_Count", i - 1)
			end

			AdjustPanel(self)
		end
	end

	local function Generate(self, index)
		if self.ElementType and index > self.Count then
			local ele

			for i = self.Count + 1, index do
				ele = self.ElementType(self.ElementPrefix .. i, self)
				ele.ID = i

				AdjustElement(ele, self)

				self:Fire("OnElementAdd", ele)

				self:SetAttribute("IFSecurePanel_Count", i)
			end

			AdjustPanel(self)
		end
	end

	local function nextItem(self, index)
		index = index + 1
		if self:GetChild(self.ElementPrefix .. index) then
			return index, self:GetChild(self.ElementPrefix .. index)
		end
	end

	_OperationMap = {
		-- 0 - AdjustPanel(self)
		KeepMaxSize = 0,
		MarginRight = 0,
		MarginBottom = 0,

		-- 1 - self:Each(AdjustElement, self)
		MarginLeft = 1,
		MarginTop = 1,
		VSpacing = 1,
		HSpacing = 1,
		Orientation = 1,
		ElementHeight = 1,
		ElementWidth = 1,

		-- 2 - SecureUpdatePanelSize(self)
		AutoSize = 2,

		-- 3 - Reduce + AdjustElement
		RowCount = 3,
		ColumnCount = 3,
	}

	local function OnPropertyChanged(self, value, old, prop)
		return Task.NoCombatCall(function()
			self:SetAttribute("IFSecurePanel_" .. prop, value)

			local oper = _OperationMap[prop]
			if oper == 1 or oper == 3 then
				if oper == 3 then Reduce(self) end
				return self:Each(AdjustElement, self)
			elseif oper == 2 then
				return SecureUpdatePanelSize(self)
			end

			return AdjustPanel(self)
		end)
	end

	class "Element"
		__Doc__[[Element is an accessor to the IFSecurePanel's elements, used like object.Element[i].Prop = value]]

		------------------------------------------------------
		-- Constructor
		------------------------------------------------------
	    function Element(self, elementPanel)
			self.__IFSecurePanel = elementPanel
	    end

		------------------------------------------------------
		-- __index
		------------------------------------------------------
		function __index(self, index)
			self = self.__IFSecurePanel

			if type(index) == "number" and index >= 1 and index <= self.ColumnCount * self.RowCount then
				index = floor(index)

				if self:GetChild(self.ElementPrefix .. index) then return self:GetChild(self.ElementPrefix .. index) end

				if self.ElementType and not InCombatLockdown() then
					Generate(self, index)

					return self:GetChild(self.ElementPrefix .. index)
				else
					return nil
				end
			end
		end
	endclass "Element"

	------------------------------------------------------
	-- Event
	------------------------------------------------------
	__Doc__[[
		<desc>Fired when an element is added</desc>
		<param name="element">System.Widget.Region, the new element that added to the panel</param>
	]]
	event "OnElementAdd"

	__Doc__[[
		<desc>Fired when an element is removed</desc>
		<param name="element">System.Widget.Region, the new element that removed from the panel</param>
	]]
	event "OnElementRemove"

	------------------------------------------------------
	-- Method
	------------------------------------------------------
	function Next(self, key)
		return nextItem, self, tonumber(key) or 0
	end

	function EachK(self, key, oper, ...)
		IFIterator.EachK(self, key, oper, ...)

		Task.NoCombatCall(AdjustPanel, self)
	end

	function Each(self, oper, ...)
		IFIterator.Each(self, oper, ...)

		Task.NoCombatCall(AdjustPanel, self)
	end

	__Doc__[[Update the panel size manually]]
	function UpdatePanelSize(self)
		Task.NoCombatCall(SecureUpdatePanelSize, self)
	end

	------------------------------------------------------
	-- Property
	------------------------------------------------------
	__Doc__[[The columns's count]]
	__Handler__( OnPropertyChanged )
	property "ColumnCount" { Type = PositiveNumber, Default = 99 }

	__Doc__[[The row's count]]
	__Handler__( OnPropertyChanged )
	property "RowCount" { Type = PositiveNumber, Default = 99 }

	__Doc__[[The elements's max count]]
	property "MaxCount" { Get = function(self) return self.ColumnCount * self.RowCount end }

	__Doc__[[The element's width]]
	__Handler__( OnPropertyChanged )
	property "ElementWidth" { Type = PositiveNumber, Default = 16 }

	__Doc__[[The element's height]]
	__Handler__( OnPropertyChanged )
	property "ElementHeight" { Type = PositiveNumber, Default = 16 }

	__Doc__[[The element's count]]
	property "Count" {
		Get = function(self)
			return self:GetAttribute("IFSecurePanel_Count")
		end,
		Set = function(self, cnt)
			if cnt > self.RowCount * self.ColumnCount then
				error("Count can't be more than "..self.RowCount * self.ColumnCount, 2)
			end

			if cnt > self.Count then
				if self.ElementType then
					Task.NoCombatCall(Generate, self, cnt)
				else
					error("ElementType not set.", 2)
				end
			elseif cnt < self.Count then
				Task.NoCombatCall(Reduce, self, cnt)
			end
		end,
		Type = NaturalNumber,
	}

	__Doc__[[The orientation for elements]]
	__Handler__( OnPropertyChanged )
	property "Orientation" { Type = Orientation, Default = Orientation.HORIZONTAL }

	__Doc__[[The element's type]]
	property "ElementType" { Type = -IFSecureHandler }

	__Doc__[[The horizontal spacing]]
	__Handler__( OnPropertyChanged )
	property "HSpacing" { Type = NaturalNumber }

	__Doc__[[The vertical spacing]]
	__Handler__( OnPropertyChanged )
	property "VSpacing" { Type = NaturalNumber }

	__Doc__[[Whether the elementPanel is autosize]]
	__Handler__( OnPropertyChanged )
	property "AutoSize" { Type = Boolean }

	__Doc__[[The top margin]]
	__Handler__( OnPropertyChanged )
	property "MarginTop" { Type = NaturalNumber }

	__Doc__[[The bottom margin]]
	__Handler__( OnPropertyChanged )
	property "MarginBottom" { Type = NaturalNumber }

	__Doc__[[The left margin]]
	__Handler__( OnPropertyChanged )
	property "MarginLeft" { Type = NaturalNumber }

	__Doc__[[The right margin]]
	__Handler__( OnPropertyChanged )
	property "MarginRight" { Type = NaturalNumber }

	__Doc__[[The Element accessor, used like obj.Element[i].]]
	property "Element" {
		Get = function(self)
			self.__IFSecurePanel_Element = self.__IFSecurePanel_Element or Element(self)
			return self.__IFSecurePanel_Element
		end,
	}

	__Doc__[[The prefix for the element's name]]
	property "ElementPrefix" { Type = System.String + nil, Default = "Element" }

	__Doc__[[Whether the elementPanel should keep it's max size]]
	__Handler__( OnPropertyChanged )
	property "KeepMaxSize" { Type = System.Boolean }

	------------------------------------------------------
	-- Event Handler
	------------------------------------------------------
	local function OnElementAdd(self, element)
		Task.NoCombatCall(RegisterFrame, self, element)
	end

	local function OnElementRemove(self, element)
		Task.NoCombatCall(UnregisterFrame, self, IGAS:GetUI(element))
	end

	------------------------------------------------------
	-- Dispose
	------------------------------------------------------
	function Dispose(self)
		self.OnElementAdd = self.OnElementAdd - OnElementAdd
		self.OnElementRemove = self.OnElementRemove - OnElementRemove

		for i = 1, self.Count do
			Task.NoCombatCall(UnregisterFrame, IGAS:GetUI(self), IGAS:GetUI(self.Element[i]))
		end

		Task.NoCombatCall(UnregisterPanel, IGAS:GetUI(self))
	end

	------------------------------------------------------
	-- Initialize
	------------------------------------------------------
	function IFSecurePanel(self)
		self.OnElementAdd = self.OnElementAdd + OnElementAdd
		self.OnElementRemove = self.OnElementRemove + OnElementRemove

		RegisterPanel(self)
	end
endinterface "IFSecurePanel"